@@ -417,6 +417,24 @@ class SQLEngineClass(BaseSQLEngineClass, PickleMixIn):
 
 
 def close_engine(engine):
+    if engine and hasattr(engine, 'cancel'):
+        try:
+            engine.cancel()
+        except:
+            pass
+
+    if engine and hasattr(engine, 'connection') and hasattr(engine.connection, 'cancel'):
+        try:
+            engine.connection.cancel()
+        except:
+            pass
+
+    if engine and hasattr(engine, 'connection') and hasattr(engine.connection, 'close'):
+        try:
+            engine.connection.close()
+        except:
+            pass
+
     if engine and hasattr(engine, 'close'):
         try:
             engine.close()
